@@ -9,6 +9,8 @@ from __future__ import absolute_import
 import os, logging
 import re
 import pysam
+from itertools import groupby, combinations as comb
+from operator import itemgetter
 from .tRNAtools import countReads, newModsParser
 from .getCoverage import getBamList, filterCoverage
 from multiprocessing import Pool
@@ -17,7 +19,8 @@ from functools import partial
 import pandas as pd
 import numpy as np
 from collections import defaultdict
-from .ssAlign import getAnticodon, clusterAnticodon, tRNAclassifier
+import subprocess
+from .ssAlign import getAnticodon, clusterAnticodon, tRNAclassifier, tRNAclassifier_nogaps
 
 log = logging.getLogger(__name__)
 
@@ -65,7 +68,7 @@ def unknownMods(inputs, out_dir, knownTable, cluster_dict, modTable, misinc_thre
 			cluster = isodecoder
 		short_isodecoder = "-".join(isodecoder.split("-")[:-1]) if not "chr" in isodecoder else isodecoder
 		anticodon = clusterAnticodon(cons_anticodon, short_isodecoder)
-		for pos, nucl in data.items():
+		for pos in data.keys():
 			cov = cov_table[isodecoder][pos]
 			if (sum(modTable[isodecoder][pos].values()) >= misinc_thresh and cov >= min_cov and pos-1 not in knownTable[cluster]): # misinc above threshold, cov above threshold and not previously known
 				# if one nucleotide dominates misinc. pattern (i.e. >= 0.9 of all misinc, likely a true SNP or misalignment)
@@ -97,7 +100,7 @@ def unknownMods(inputs, out_dir, knownTable, cluster_dict, modTable, misinc_thre
 
 	return(new_mods_cluster, new_inosines_cluster)
 
-def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict, cca, tRNA_struct, remap, misinc_thresh, knownTable, tRNA_dict, unique_isodecoderMMs, splitBool, isodecoder_sizes, threads, inputs):
+def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, del_dict, cluster_dict, cca, tRNA_struct, remap, misinc_thresh, knownTable, tRNA_dict, unique_isodecoderMMs, splitBool, isodecoder_sizes, threads, inputs):
 # modification counting and table generation, and CCA analysis
 	
 	modTable = defaultdict(lambda: defaultdict(lambda: defaultdict(int)))
@@ -106,7 +109,6 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 	cov = defaultdict(lambda: defaultdict(int))
 	geneCov = defaultdict(int)
 	condition = info[inputs][0]
-
 	# initialise structures and outputs if CCA analysis in on
 	if cca:
 		aln_count = 0
@@ -119,7 +121,6 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 	bam_file = pysam.AlignmentFile(inputs, "rb")
 	log.info('Analysing {}...'.format(inputs))
 	for read in bam_file.fetch(until_eof=True):
-		query = read.query_name
 		reference = read.reference_name
 
 		#########################
@@ -150,12 +151,21 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 			del cigar_list[-3:-1]
 
 		# remove insertions in read from the sequence (MD tags do not account for deletions and effect misinc. identity matching)
+		# record ref_deletions (insertions in read) for matching later to unique differences between ref and read for isodecoder splitting - note use read_pos here so that deletions in read_seq are not subtracted from insert positions
 		read_ins_pos = 0
+		read_pos = 0
+		ref_deletions = list()
 		for index, i in enumerate(cigar_list):
 			if i.isdigit():
 				read_ins_pos += int(i)
+				read_pos += int(i)
+			elif i.isalpha() and i.upper() == 'D':
+				del_size = int(cigar_list[index-1])
+				read_ins_pos -= del_size
+				read_pos -= del_size
 			elif i.isalpha() and i.upper() == 'I':
 				ins_size = int(cigar_list[index-1])
+				ref_deletions.append(read_pos-ins_size)
 				read_seq = read_seq[0:read_ins_pos-ins_size] + read_seq[read_ins_pos:]
 				read_ins_pos -= ins_size
 
@@ -164,19 +174,32 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 		offset = read.reference_start
 		aln_end = read.reference_end
 		
-		# map mismatches to reference and count mods
+		# count mods and find new reference
 		ref_pos = 0
 		read_pos = 0
+		adjust = 0
 		temp = defaultdict()
-		temp, ref_pos, read_pos, reference = countMods(temp, ref_pos, read_pos, read_seq, offset, reference, md_list, unique_isodecoderMMs, mismatch_dict, insert_dict, remap)
-
+		temp, ref_pos, read_pos, readRef_dif, insertions = countMods(temp, reference, ref_pos, read_pos, read_seq, offset, md_list, ref_deletions, tRNA_dict, mismatch_dict, insert_dict, del_dict, remap)
+		if readRef_dif: # only assign new reference if readRef_dif is recorded which only happend when remap = False (i.e. after 2nd alignment or if remap is never activated)
+			reference, temp, adjust = findNewReference(unique_isodecoderMMs, splitBool, readRef_dif, reference, temp, insertions, insert_dict, del_dict, ref_deletions, adjust)
 		# read counts, stops and coverage
 		counts[inputs][reference] += 1
 		geneCov[reference] += 1
+		
 		# offset + 1 (0 to 1 based) is start of alignment - i.e. any start > 1 indicates a stop to RT at this position
-		stopTable[reference][offset+1] += 1
-		for i in range(offset+1, aln_end+1):
-			cov[reference][i] += 1
+		# correct for members that are shorter than parents at 5' end using adjust variable (see countMods)
+		
+		# only for reads that start at the 0 position of memebers they are assigned to
+		# there are weird cases where a read is longer at 5' end than its new assigned member (probably incorrect assignment) and this would generate negative values for stop
+		if offset - adjust >= 0:
+			stopTable[reference][offset+1] += 1
+			for i in range(offset+1, aln_end+1):
+				cov[reference][i] += 1
+		# if it is a weird case as described above, then just assume the read is full-length and add to stop at position 1
+		else:
+			stopTable[reference][1] += 1
+			for i in range(0, aln_end+1):
+				cov[reference][i] += 1
 
 		for pos, identity in temp.items():
 			modTable[reference][pos][identity] += 1
@@ -198,7 +221,7 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 			elif ref_pos == ref_length - 2:
 				dinuc = read.query_sequence[-1:]
 				cca_dict[reference][dinuc] += 1
-			elif ref_pos == ref_length - 3:
+			elif ref_pos <= ref_length - 3:
 				dinuc = "Absent"
 				cca_dict[reference][dinuc] += 1
 
@@ -236,8 +259,8 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 	
 	# format and output mods and stops if remap is disabled (i.e. also occurs after round 2 of mapping)
 	if not remap:
-		new_mods = {}
-		new_Inosines = {}
+	#	new_mods = {}
+	#	new_Inosines = {}
 
 		log.info('Building modification, stop and count data tables for {}'.format(inputs))
 
@@ -249,11 +272,12 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 		modTable_prop_melt['condition'] = condition
 		modTable_prop_melt['bam'] = inputs
 		modTable_prop_melt.pos = pd.to_numeric(modTable_prop_melt.pos)
+		#modTable_prop_melt.to_csv("premismatchTable.csv", sep = "\t", index = False, na_rep = 'NA')
 
 		# split and parallelize addNA
 		names, dfs = splitTable(modTable_prop_melt)
 		pool = Pool(threads)
-		func = partial(addNA, tRNA_struct, cluster_dict, "mods")
+		func = partial(addNA, tRNA_struct, "mods")
 		modTable_prop_melt = pd.concat(pool.starmap(func, zip(names, dfs)))
 		pool.close()
 		pool.join()
@@ -271,7 +295,7 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 		cov_table_na = cov_table_melt.copy()
 		names, dfs = splitTable(cov_table_na)
 		pool = Pool(threads)
-		func = partial(addNA, tRNA_struct, cluster_dict, "cov")
+		func = partial(addNA, tRNA_struct, "cov")
 		cov_table_na = pd.concat(pool.starmap(func, zip(names, dfs)))
 		pool.close()
 		pool.join()
@@ -297,7 +321,7 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 		stopTable_prop_melt.dropna(inplace = True)
 		names, dfs = splitTable(stopTable_prop_melt)
 		pool = Pool(threads)
-		func = partial(addNA, tRNA_struct, cluster_dict, "stops")
+		func = partial(addNA, tRNA_struct, "stops")
 		stopTable_prop_melt = pd.concat(pool.starmap(func, zip(names, dfs)))
 		pool.close()
 		pool.join()
@@ -316,7 +340,7 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 		readthroughTable_melt.dropna(inplace = True)
 		names, dfs = splitTable(readthroughTable_melt)
 		pool = Pool(threads)
-		func = partial(addNA, tRNA_struct, cluster_dict, "stops")
+		func = partial(addNA, tRNA_struct, "stops")
 		readthroughTable_melt = pd.concat(pool.starmap(func, zip(names, dfs)))
 		pool.close()
 		pool.join()
@@ -353,7 +377,7 @@ def bamMods_mp(out_dir, min_cov, info, mismatch_dict, insert_dict, cluster_dict,
 			# write CCA outputs for current bam
 			for cluster, data in cca_dict.items():
 					for dinuc, count in data.items():
-						if (dinuc.upper() == "CC") or (dinuc.upper() == "CA") or (dinuc.upper() == "C") or (dinuc == "Absent"):
+						if dinuc.upper() in ["CA", "CC", "C", "ABSENT"]:
 							CCAvsCC_counts.write(cluster + "\t" + dinuc + "\t" + inputs + "\t" + condition + "\t" + str(count) + "\n")
 
 			dinuc_prop.close()
@@ -372,11 +396,12 @@ def splitTable(table):
 
 	return(names, dfs)
 
-def countMods(temp, ref_pos, read_pos, read_seq, offset, reference, md_list, unique_isodecoderMMs, mismatch_dict, insert_dict, remap):
-# Loop though mismatches in read and return new reference for splitting reads and/or count mods
+def countMods(temp, reference, ref_pos, read_pos, read_seq, offset, md_list, ref_deletions, tRNA_dict, mismatch_dict, insert_dict, del_dict, remap):
+# Loop though mismatches in read, assign to new deconvoluted reference (if possible) and count mods
 	
-	old_reference = reference
 	insertions = list()
+	#deletions = list()
+	readRef_dif = tuple() # mismatches, insertions and deleteion in read relative to reference used to assign to isodecoders
 	for index, interval in enumerate(md_list):
 		if not index == 0:
 			new_offset = 0
@@ -390,40 +415,110 @@ def countMods(temp, ref_pos, read_pos, read_seq, offset, reference, md_list, uni
 			elif interval.isalpha(): # is a mismatch
 				identity = read_seq[read_pos]
 				ref_pos += new_offset 
-				# update reference for isodecoder splitting if cluster_id not 1 and remap is disabed or this is round 2 of alignment (avoid errors in adding new mods for clusters)
-				if (unique_isodecoderMMs) and (identity.upper() in unique_isodecoderMMs[old_reference][ref_pos]) and (not remap):
-					reference = unique_isodecoderMMs[old_reference][ref_pos][identity]
-				# only include these positions if they aren't registered mismatches between clusters, or if they are known modified sites (lowercase)
-				elif (ref_pos not in mismatch_dict[old_reference]): #or (ref_pos in mismatch_dict[old_reference] and interval.islower()):
+				# check if current mismatch is in cluster mismatches and add to tuple of differences for deconvolution
+				# if cluster_id not 1 and remap is disabed or this is round 2 of alignment (avoid errors in adding new mods for clusters)
+				if (ref_pos in mismatch_dict[reference]) and (not remap) and (not ref_pos in tRNA_dict[reference]['modified']):
+					toAdd = str(ref_pos) + identity
+					readRef_dif = readRef_dif + (toAdd,)				
+				# only include these positions if they aren't registered mismatches between clusters
+				elif (ref_pos not in mismatch_dict[reference]):
 					temp[ref_pos+1] = identity
 				# move forward
 				read_pos += 1
 				ref_pos += 1
 		elif interval.startswith('^'):
-			identity = 'insertion'
+			identity = 'Ins'
+			insert_length = len(interval) - 1
 			#insertions.extend([x + ref_pos for x in range(len(interval) - 1)]) # register all insertions (including consecutive insertions) to be checked later against cluster parent
-			insertions.append(ref_pos)
-			if (unique_isodecoderMMs) and (identity in unique_isodecoderMMs[old_reference][ref_pos]) and (not remap):
-				reference = unique_isodecoderMMs[old_reference][ref_pos][identity]
-			insertion = len(interval) - 1
-			ref_pos += insertion
+			current_inserts = [x for x in range(ref_pos,ref_pos+insert_length)]
+			insertions.extend(current_inserts)
+			for ins in current_inserts:
+				if (ins in insert_dict[reference].keys()) and (not remap): # only add position to tuple to deconvolute if it exists as a known difference in the cluster
+					toAdd = str(ref_pos) + identity
+					readRef_dif = readRef_dif + (toAdd,)
+			ref_pos += insert_length
 
-	# handle insertions between cluster parent and members present in read (different from insertions in read only)
-	# i.e. once new ref is found above and this member has an insertion relative to parent, subtract 1 from all misinc positions after the insertion
+	# add applicable deletions after cycling through MD list
+	identity = "Del"
+	for deletion in ref_deletions:
+		if deletion in del_dict[reference].keys() and (not remap):
+			toAdd = str(deletion) + identity
+			readRef_dif = readRef_dif + (toAdd,)
+
+	return(temp, ref_pos, read_pos, readRef_dif, insertions)
+
+def findNewReference(unique_isodecoderMMs, splitBool, readRef_dif, reference, temp, insertions, insert_dict, del_dict, ref_deletions, adjust):
+# function to find new reference for read based on mismatches to cluster parent
+
+	old_reference = reference
+	# Check sorted readRef_dif in unique_isodecoderMMs to update reference
+	readRef_dif = tuple(sorted(readRef_dif))
+	if readRef_dif:
+		if readRef_dif in unique_isodecoderMMs[old_reference].keys():
+			# set new reference only if it is not in splitBool - these are unsplit isodecoders because of significant cov difference between 3' end and mismatch used for splitting
+			reference = unique_isodecoderMMs[old_reference][readRef_dif][0]
+		# if it is not found, it may be some shorter combination as unique_isodecoderMMs contains shortest possible combination set
+		else:
+			intersectLen = dict()
+			for uss in unique_isodecoderMMs[old_reference].keys():
+				uss_set = set(uss) 
+				intersection = uss_set.intersection(readRef_dif)
+				intersectLen[uss] = len(intersection)
+			
+			maxIntersect = max(intersectLen.values())
+			matches = [match for match, length in intersectLen.items() if length == maxIntersect and not length == 0]
+			if len(matches) == 1:
+				reference = unique_isodecoderMMs[old_reference][matches[0]][0]
+			elif len(matches) > 1:
+				diffLen = dict()
+				for match in matches:
+					diff = len(set(match) - set(readRef_dif))
+					diffLen[diff] = match
+				minuss = diffLen[min(diffLen.keys())]
+				reference = unique_isodecoderMMs[old_reference][minuss][0]
+
+	# handle insertions and deletions between cluster parent and members present in read (different from insertions or deletions in read only)
+	# i.e. once new ref is found above and this member has an insertion or deletion relative to parent, subtract (ins) or add (del) 1 from all misinc positions after the insertion
 	# corrects for difference in length between member and parent. 
-	# Note that 1 bp up and down from the insertion are checked to account for differences in insertion position due to short read aligner, and usearch clustering
-	if (not reference == old_reference) and (insertions):
+	# Note that 1 bp up and down from the insertion/deletion are checked to account for differences in insertion/deletion position due to short read aligner and usearch clustering
+	if (not reference == old_reference):
 		for i in insertions:
 			if reference in (insert_dict[old_reference][i] or insert_dict[old_reference][i+1] or insert_dict[old_reference][i-1]):
 				temp = {(k - 1 if k > i else k):v for k,v in temp.items()}
+		for d in ref_deletions:
+			if reference in (del_dict[old_reference][d] or del_dict[old_reference][d+1] or del_dict[old_reference][d-1]):
+				temp = {(k + 1 if k > d else k):v for k, v in temp.items()}
+			
+		# special case for members that are shorter than parents at the 5' end or 3'
+		# need to subtract the number of bases from recorded mismatches in temp to correct the position info
+		cluster_inserts = [ins for ins in insert_dict[old_reference].keys() if reference in insert_dict[old_reference][ins]] # get all inserts in parent for the new reference (child)
+		consec_inserts_5 = list()
+		if cluster_inserts:
+			for k, g in groupby(enumerate(cluster_inserts), lambda ix: ix[0] - ix[1]): # this gets lists of consecutive inserts - non-consecutive inserts could just be normal inserts in the body
+				l = list(map(itemgetter(1), g))
+				if 0 in l: # checks consecutive inserts for those starting at the 5' end of the parent
+					consec_inserts_5 = l
+					adjust = max(consec_inserts_5) #+ 1 # add one to adjustment variable because an insert at (for e.g.) pos 0 of the parent indicates that the member starts at position 1 of the parent
+		temp = {(k - adjust):v for k,v in temp.items() if k >= adjust}
 
-	return(temp, ref_pos, read_pos, reference)
+	#	cluster_deletions = [deletion for deletion in del_dict[old_reference].keys() if reference in del_dict[old_reference][deletion]] # get all inserts in parent for the new reference (child)
+	#	consec_dels_5 = list()
+	#	if cluster_deletions:
+	#		for k, g in groupby(enumerate(cluster_deletions), lambda ix: ix[0] - ix[1]): # this gets lists of consecutive inserts - non-consecutive inserts could just be normal inserts in the body
+	#			l = list(map(itemgetter(1), g))
+				#if 0 in l: # checks consecutive inserts for those starting at the 5' end of the parent
+				#	consec_inserts_5 = l
+				#	adjust = max(consec_inserts_5) #+ 1 # add one to adjustment variable because an insert at (for e.g.) pos 0 of the parent indicates that the member starts at position 1 of the parent
+		#temp = {(k - adjust):v for k,v in temp.items() if k >= adjust}
 
-def addNA(tRNA_struct, cluster_dict, data_type, name, table):
+	return(reference, temp, adjust)
+
+def addNA(tRNA_struct, data_type, name, table):
 # fill mods and stops tables with 'NA' for gapped alignment
-
+	
 	#cluster = [parent for parent, child in cluster_dict.items() if name in child][0]
 	shortname = "-".join(name.split("-")[:-1]) if not "chr" in name else name
+	new = pd.DataFrame()
 	for pos in tRNA_struct.loc[shortname].index:
 		if data_type == 'mods':
 			#new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'type':pd.Categorical(['A','C','G','T']), 'proportion':'NA', 'condition':table.condition.iloc[1], 'bam':table.bam.iloc[1], 'cov':'NA'})
@@ -432,7 +527,7 @@ def addNA(tRNA_struct, cluster_dict, data_type, name, table):
 			new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'proportion':'NA', 'condition':table.condition.iloc[0], 'bam':table.bam.iloc[0]}, index=[0])
 		elif data_type == 'cov':
 			new = pd.DataFrame({'isodecoder':name, 'pos':pos, 'bam':table.bam.iloc[0], 'cov':'NA'}, index=[0])
-		if tRNA_struct.loc[shortname].iloc[pos-1].struct == 'gap':
+		if tRNA_struct.loc[(shortname, pos)].struct == 'gap':
 			if not pos == max(tRNA_struct.loc[shortname].index):
 				table.loc[(table.isodecoder == name) & (table.pos >= pos), 'pos'] += 1
 			#if not data_type == "readthrough":
@@ -442,7 +537,7 @@ def addNA(tRNA_struct, cluster_dict, data_type, name, table):
 
 	return(table)
 
-def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_dict, insert_dict, cluster_dict, cca, remap, misinc_thresh, knownTable, Inosine_lists, tRNA_dict, Inosine_clusters, unique_isodecoderMMs, splitBool, isodecoder_sizes, clustering):
+def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_dict, insert_dict, del_dict, cluster_dict, cca, remap, misinc_thresh, knownTable, Inosine_lists, tRNA_dict, Inosine_clusters, unique_isodecoderMMs, splitBool, isodecoder_sizes, clustering):
 # Wrapper function to call countMods_mp with multiprocessing
 
 	if cca:
@@ -480,25 +575,35 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 		multi = len(baminfo)
 
 	# get tRNA struct info from ssAlign
-	tRNA_struct, tRNA_ungap2canon, cons_pos_list, cons_pos_dict = tRNAclassifier(out_dir)
+	tRNA_struct, tRNA_ungap2canon = tRNAclassifier()[0:2]
+	cons_pos_dict = tRNAclassifier()[3]
+
 	tRNA_struct_df = pd.DataFrame(tRNA_struct).unstack().rename_axis(('cluster', 'pos')).rename('struct')
 	tRNA_struct_df = pd.DataFrame(tRNA_struct_df)
 
+	# format canonical position dictioary for merging with various tables below
+	tRNA_ungap2canon_table = pd.DataFrame.from_dict(tRNA_ungap2canon, orient = "index")
+	tRNA_ungap2canon_table = tRNA_ungap2canon_table.reset_index()
+	tRNA_ungap2canon_table = tRNA_ungap2canon_table.melt(var_name='pos', value_name='canon_pos', id_vars='index')
+	tRNA_ungap2canon_table.columns = ['isodecoder', 'pos', 'canon_pos']
+	tRNA_ungap2canon_table['pos'] = tRNA_ungap2canon_table['pos'].astype(int)
+
 	# initiate custom non-daemonic multiprocessing pool and run with bam names
 	pool = MyPool(multi)
 	# to avoid assigning too many threads, divide available threads by number of processes
 	threadsForMP = int(threads/multi)
-	func = partial(bamMods_mp, out_dir, min_cov, baminfo, mismatch_dict, insert_dict, cluster_dict, cca, tRNA_struct_df, remap, misinc_thresh, knownTable, tRNA_dict, unique_isodecoderMMs, splitBool, isodecoder_sizes, threadsForMP)
+	func = partial(bamMods_mp, out_dir, min_cov, baminfo, mismatch_dict, insert_dict, del_dict, cluster_dict, cca, tRNA_struct_df, remap, misinc_thresh, knownTable, tRNA_dict, unique_isodecoderMMs, splitBool, isodecoder_sizes, threadsForMP)
 	new_mods, new_Inosines = zip(*pool.map(func, bamlist))
 	pool.close()
 	pool.join()
 
 	filtered = list()
+	filter_warning = False
 	
 	if not remap:
 
 		# Redo newModsParser here so that knownTable is updated with new mods from second round and written to allModsTable
-		Inosine_clusters = newModsParser(out_dir, name, new_mods, new_Inosines, knownTable, Inosine_lists, tRNA_dict, clustering)
+		Inosine_clusters, snp_tolerance, newtRNA_dict, newknownTable = newModsParser(out_dir, name, new_mods, new_Inosines, knownTable, Inosine_lists, tRNA_dict, clustering, remap, snp_tolerance = True)
 
 		modTable_total = pd.DataFrame()
 		countsTable_total = pd.DataFrame()
@@ -508,16 +613,17 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 
 		dinuc_table = pd.DataFrame()
 		CCAvsCC_table = pd.DataFrame()
-
+ 
 		for bam in bamlist:
 			# read in temp files and then delete
 			modTable = pd.read_csv(bam + "mismatchTable.csv", header = 0, sep = "\t")
 			modTable['canon_pos'] = modTable['pos'].map(cons_pos_dict)
-			# edit misinc. propoportions of inosines to reflect true level of Gs
+			# edit misinc. propoportions of inosines to reflect true level of Gs, set As to NA
 			for cluster in Inosine_clusters:
 				for isodecoder in cluster_dict[cluster]:
 					if any(modTable.isodecoder.str.contains(isodecoder)):
 						modTable.at[(modTable.canon_pos == '34') & (modTable['type'] == 'G') & (modTable.isodecoder == isodecoder), 'proportion'] = 1 - sum(modTable[(modTable.canon_pos == '34') & (modTable['type'] != 'G') & (modTable.isodecoder == isodecoder)]['proportion'].dropna())
+						modTable.at[(modTable.canon_pos == '34') & (modTable['type'] == 'A') & (modTable.isodecoder == isodecoder), 'proportion'] = np.nan
 			os.remove(bam + "mismatchTable.csv")
 
 			stopTable = pd.read_csv(bam + "RTstopTable.csv", header = 0, sep = "\t")
@@ -558,17 +664,20 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 		countsTable_total.loc[~countsTable_total['isodecoder'].str.contains("chr"), 'isodecoder'] = countsTable_total['isodecoder'].str.split("-").str[:-1].str.join("-")
 		countsTable_total.index = countsTable_total.isodecoder
 		countsTable_total.drop(columns = ['isodecoder'], inplace = True)
-		filtered = filterCoverage(countsTable_total, min_cov)
+		filtered, filter_warning = filterCoverage(countsTable_total, min_cov)
+
+		# edit splitBool isodecoder names for filtering from tables and adding to Single_isodecoder in counts files
+		splitBool = ["-".join(x.split("-")[:-1]) for x in splitBool]
 
 		# filter and output tables
 		modTable_total.loc[~modTable_total['isodecoder'].str.contains("chr"), 'isodecoder'] = modTable_total['isodecoder'].str.split("-").str[:-1].str.join("-")
 		modTable_total = modTable_total[~modTable_total.isodecoder.isin(filtered)]
-		#modTable_total.drop(modTable_total[modTable_total['isodecoder'] in filtered].index, inplace = True)
+		modTable_total = modTable_total[~modTable_total.isodecoder.isin(splitBool)]
 		modTable_total.drop_duplicates(inplace = True)
 		modTable_total.to_csv(out_dir + "mods/mismatchTable.csv", sep = "\t", index = False, na_rep = 'NA')
 		with open(out_dir + "mods/allModsTable.csv", "w") as known:
 			known.write("cluster\tpos\n")
-			for cluster, data in knownTable.items():
+			for cluster, data in newknownTable.items():
 				for pos in data:
 					known.write("-".join(cluster.split("-")[:-1]) + "\t" + str(pos+1) + "\n")
 
@@ -582,21 +691,33 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 		readthroughTable_total.drop_duplicates(inplace = True)
 		readthroughTable_total.to_csv(out_dir + "mods/readthroughTable.csv", sep = "\t", index = False, na_rep = 'NA')	
 		
-		# add column to counts to indicate complete isodecoder split or not
+		# add column to counts to indicate complete isodecoder split or not, sizes, and parent
 		countsTable_total['Single_isodecoder'] = "NA"
+		isodecoder_sizes_short = defaultdict()
+		for iso, size in isodecoder_sizes.items():
+			if not "chr" in iso:
+				short = "-".join(iso.split("-")[:-1])
+			else:
+				short = iso
+			isodecoder_sizes_short[short] = size
 		for cluster in countsTable_total.index:
 			if cluster in splitBool:
 				countsTable_total.at[cluster, 'Single_isodecoder'] = "False"
 			else:
 				countsTable_total.at[cluster, 'Single_isodecoder'] = "True"
+			countsTable_total.at[cluster, 'size'] = isodecoder_sizes_short[cluster]
+		clusterInfo = pd.read_csv(out_dir + "/" + name + "clusterInfo.txt", sep = "\t")
+		clusterInfo = clusterInfo[['tRNA','parent']]
+		clusterInfo.columns = ['isodecoder','parent']
+		clusterInfo.loc[~clusterInfo['isodecoder'].str.contains("chr"), 'isodecoder'] = clusterInfo['isodecoder'].str.split("-").str[:-1].str.join("-")
+		clusterInfo.loc[~clusterInfo['parent'].str.contains("chr"), 'parent'] = clusterInfo['parent'].str.split("-").str[:-1].str.join("-")
+		clusterInfo.drop_duplicates(inplace=True)
+		clusterInfo.index = clusterInfo.isodecoder
+		clusterInfo.drop(columns = ['isodecoder'], inplace = True)
+		countsTable_total = countsTable_total.join(clusterInfo)
 		countsTable_total.to_csv(out_dir + "Isodecoder_counts.txt", sep = "\t", index = True, na_rep = "0")
 
 		# map canon_pos for each isodecoder ungapped pos to newMods
-		tRNA_ungap2canon_table = pd.DataFrame.from_dict(tRNA_ungap2canon, orient = "index")
-		tRNA_ungap2canon_table = tRNA_ungap2canon_table.reset_index()
-		tRNA_ungap2canon_table = tRNA_ungap2canon_table.melt(var_name='pos', value_name='canon_pos', id_vars='index')
-		tRNA_ungap2canon_table.columns = ['isodecoder', 'pos', 'canon_pos']
-		tRNA_ungap2canon_table['pos'] = tRNA_ungap2canon_table['pos'].astype(int)
 		newMods_total.loc[~newMods_total['isodecoder'].str.contains("chr"), 'isodecoder'] = newMods_total['isodecoder'].str.split("-").str[:-1].str.join("-")
 		newMods_total = pd.merge(newMods_total, tRNA_ungap2canon_table, on = ['isodecoder', 'pos'], how = "left")
 		newMods_total = newMods_total[~newMods_total.isodecoder.isin(filtered)]
@@ -619,8 +740,21 @@ def generateModsTable(sampleGroups, out_dir, name, threads, min_cov, mismatch_di
 			CCAvsCC_table.to_csv(out_dir + "CCAanalysis/CCAcounts.csv", sep = "\t", index = False)
 
 		# Anticodon and/or isodecoder counts counts
-		countReads(out_dir + "Isodecoder_counts.txt", out_dir, isodecoder_sizes, clustering, tRNA_dict)
+		countReads(out_dir + "Isodecoder_counts.txt", out_dir, isodecoder_sizes, clustering, newtRNA_dict, clusterInfo)
 
 		log.info("** Read counts per isodecoder saved to " + out_dir + "counts/Isodecoder_counts.txt **")
 
-	return(new_mods, new_Inosines, filtered)
+	return(new_mods, new_Inosines, filtered, filter_warning)
+
+def plotCCA(out_dir, double_cca):
+
+	log.info("\n+-----------------+\
+		\n| 3'-CCA analysis |\
+		\n+-----------------+")
+
+	out = out_dir + "CCAanalysis/"
+	script_path = os.path.dirname(os.path.realpath(__file__))
+	command = ["Rscript", script_path + "/ccaPlots.R", out + "AlignedDinucProportions.csv", out + "/CCAcounts.csv", out, str(double_cca)]
+	subprocess.check_call(command)
+
+	log.info("CCA analysis done and plots created. Located in {}".format(out))
